SystemOrganization addCategory: #'Ma-Client-Server-Tester-API'!
SystemOrganization addCategory: #'Ma-Client-Server-Tester-Exceptions'!
SystemOrganization addCategory: #'Ma-Client-Server-Tester-Info'!
SystemOrganization addCategory: #'Ma-Client-Server-Tester-Network testing'!
SystemOrganization addCategory: #'Ma-Client-Server-Tester-SUnit extensions'!
SystemOrganization addCategory: #'Ma-Client-Server-Tester-private'!
SystemOrganization addCategory: #'Ma-Client-Server-Tester-Object Layer Tests'!
SystemOrganization addCategory: #'Ma-Client-Server-Tester-Test cases'!

TestCase subclass: #MaTestCase
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-SUnit extensions'!

!MaTestCase commentStamp: '<historical>' prior: 0!
Getting ready to try using "fixtureSpecs" so we can test against hundreds of fixtures easily.
!

MaTestCase subclass: #MaNetworkTestCase
	instanceVariableNames: ''
	classVariableNames: 'NextPort PlayerLinks TestPlayer'
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Network testing'!

!MaNetworkTestCase commentStamp: '<historical>' prior: 0!
My subclasses are intended to test network programs.  I provide methods for communicating with other images and tunnelling requests to them.!

MaNetworkTestCase subclass: #MaClientServerTestCase
	instanceVariableNames: 'server busy mergedBag testObjects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Test cases'!

!MaClientServerTestCase class methodsFor: 'accessing' stamp: 'cmm 2/7/2009 10:56'!
defaultServerPort
	^ 60948! !

!MaClientServerTestCase class methodsFor: 'initialize-release' stamp: 'cmm 2/7/2009 10:04'!
initialize
	super initialize.
	Smalltalk addToStartUpList: self! !

!MaClientServerTestCase class methodsFor: 'required overrides' stamp: 'cmm 2/6/2009 12:49'!
setUpPlayers
	self
		addPlayerNamed: 'client1';
		addPlayerNamed: 'client2';
		addPlayerNamed: 'server'! !

!MaClientServerTestCase class methodsFor: 'required overrides' stamp: 'cmm 2/6/2009 17:19'!
suiteName
	^ 'macs'! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 11/10/2008 11:28'!
newBagFrom: start to: stop 
	^ ((start to: stop) collect: [ : n | n asWords ]) asBag! !

!MaClientServerTestCase methodsFor: 'shared' stamp: 'cmm 11/10/2008 15:47'!
noteBusyWhile: aBlock 
	
	[ busy := true.
	aBlock value ] ensure: [ busy := false ]! !

!MaClientServerTestCase methodsFor: 'test object server' stamp: 'cmm 11/9/2008 11:35'!
objectProcessorBlock
	^ [ : aMacsTestRequest | MacsTestResponse fromRequest: aMacsTestRequest ]! !

!MaClientServerTestCase methodsFor: 'armored code' stamp: 'cmm 11/10/2008 14:52'!
playerProtocol
	^ super playerProtocol , self testServerObjectProtocol! !

!MaClientServerTestCase methodsFor: 'test UDP object server' stamp: 'cmm 11/11/2008 14:44'!
sendObjectsToUdpServer: aCollection 
	| myLink |
	myLink := MaNetworkServerLink location: (MaServerLocation 
			host: (self class getHostNameFor: 'server')
			port: self class defaultServerPort).
	myLink
		networkProtocol: #udp ;
		protocol: self testServerObjectProtocol.
	[ self noteBusyWhile: [ aCollection do: [ : each | myLink submit: each ] ] ] fork! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 11/11/2008 16:32'!
sendOutUdpByteArrayRequestsFrom: start to: stop 
	"Send the server the numbers, in English words, from 500 to 1000."
	| bag mySocket |
	bag := self 
		newBagFrom: start
		to: stop.
	mySocket := MaClientSocket location: (MaServerLocation 
			host: (self class getHostNameFor: 'server')
			port: self class defaultServerPort).
	mySocket networkProtocol: #udp.
	[ busy := true.
	bag do: 
		[ : each | 
		mySocket 
			sendData: each
			startingAt: 2
			count: each size - 2
			waitForReplyIn: nil
		"		(Delay forMilliseconds: 10 atRandom) wait" ].
	busy := false ] fork! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 11/8/2008 17:42'!
startByteArrayRequests
	| mySocket requestByteArray response eachResponse randomStream byteArrayRandomizer requestSize largestTest |
	largestTest := 9000.
	randomStream := Random new.
	response := ByteArray new: 4000.	"<- purposely too small"
	requestByteArray := ByteArray new: largestTest.
	byteArrayRandomizer := 
	[ : size | 
	1 
		to: size
		do: 
			[ : index | 
			requestByteArray 
				at: index
				put: (randomStream nextInt: 256) - 1 ] ].
	mySocket := MaClientSocket location: 
		(MaServerLocation 
			host: (self class getHostNameFor: 'server')
			port: self class defaultServerPort).
	
	[ busy := true.
	1 
		to: 200
		do: 
			[ : count | 
			requestSize := randomStream nextInt: largestTest.
			byteArrayRandomizer value: requestSize.
			eachResponse := mySocket 
				sendData: requestByteArray
				startingAt: 1
				count: requestSize
				waitForReplyIn: response.
			self should: 
				[ (requestByteArray 
					copyFrom: 1
					to: requestSize) reverse = (eachResponse 
						copyFrom: 1
						to: requestSize) ] ].
	busy := false ] fork! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 11/10/2008 16:05'!
startByteArrayServer
	server := MaServerSocket new.
	server 
		processOn: self class defaultServerPort
		using: [ : requestLifecycle | requestLifecycle requestByteArray reverse ]! !

!MaClientServerTestCase methodsFor: 'test object server' stamp: 'cmm 11/10/2008 16:01'!
startObjectRequests
	| myLink |
	myLink := MaNetworkServerLink location: 
		(MaServerLocation 
			host: (self class getHostNameFor: 'server')
			port: self class defaultServerPort
"			interface: #udp").
	[ busy := true.
	200 timesRepeat: 
		[ | eachRequest myResponse remoteResponse |
		eachRequest := MacsTestRequest new.
		remoteResponse := myLink submit: eachRequest.
		myResponse := self objectProcessorBlock value: eachRequest.
		self assert: [ remoteResponse ~~ myResponse ].
		self should: [ remoteResponse response1 = myResponse response1 ].
		self should: [ remoteResponse response2 = myResponse response2 ] ].
	busy := false ] fork! !

!MaClientServerTestCase methodsFor: 'test object server' stamp: 'cmm 11/10/2008 16:01'!
startObjectServer
	server := MaNetworkRequestServer protocol: self testServerObjectProtocol.
	server 
		processOn: self class defaultServerPort
		using: self objectProcessorBlock! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 11/11/2008 16:27'!
startUdpByteArrayServer
	server := MaServerSocket new
		networkProtocol: #udp ;
		yourself.
	"Server will load this Bag."
	mergedBag := Bag new.
	server 
		processOn: self class defaultServerPort
		using: [ : requestLifecycle | mergedBag add: requestLifecycle requestByteArray asString ]! !

!MaClientServerTestCase methodsFor: 'test UDP object server' stamp: 'cmm 11/11/2008 16:27'!
startUdpObjectServer
	server := MaNetworkRequestServer protocol: self testServerObjectProtocol.
	server networkProtocol: #udp.
	testObjects := Bag new.
	server 
		processOn: self class defaultServerPort
		using: [ : eachMacsTestRequest | testObjects add: eachMacsTestRequest ]! !

!MaClientServerTestCase methodsFor: 'shared' stamp: 'cmm 11/9/2008 11:15'!
stopServer
	server shutdown! !

!MaClientServerTestCase methodsFor: 'test ByteArray server' stamp: 'cmm 11/9/2008 11:15'!
testByteArrayLayer
	self
		remotePerform: #startByteArrayServer
			in: 'server' ;
		remotePerform: #startByteArrayRequests
			in: 'client1' ;
		remotePerform: #startByteArrayRequests
			in: 'client2' ;
		remotePerform: #waitUntilDone
			in: 'client1' ;
		remotePerform: #waitUntilDone
			in: 'client2'.
	self 
		remotePerform: #stopServer
		in: 'server'! !

!MaClientServerTestCase methodsFor: 'test object server' stamp: 'cmm 11/9/2008 17:25'!
testObjectLayer
	self
		remotePerform: #startObjectServer
			in: 'server' ;
		remotePerform: #startObjectRequests
			in: 'client1' ;
		remotePerform: #startObjectRequests
			in: 'client2' ;
		remotePerform: #waitUntilDone
			in: 'client1' ;
		remotePerform: #waitUntilDone
			in: 'client2'.
	self 
		remotePerform: #stopServer
		in: 'server'! !

!MaClientServerTestCase methodsFor: 'shared' stamp: 'cmm 11/10/2008 14:51'!
testServerObjectProtocol
	^ { 
		MacsTestRequest.
		DateAndTime.
		Duration.
		Association.
		UUID.
		"String not needed, it is supported automatically."
		MacsTestResponse
	 }! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 11/10/2008 11:32'!
testUdpByteArrayLayer
	self
		remotePerform: #startUdpByteArrayServer
			in: 'server' ;
		remotePerform: #sendOutUdpByteArrayRequestsFrom:to:
			withArguments: #(500 1000 )
			in: 'client1' ;
		remotePerform: #sendOutUdpByteArrayRequestsFrom:to:
			withArguments: #(750 1250 )
			in: 'client2' ;
		remotePerform: #waitUntilDone
			in: 'client1' ;
		remotePerform: #waitUntilDone
			in: 'client2' ;
		remotePerform: #verifyMergedBag
			in: 'server' ;
		remotePerform: #stopServer
			in: 'server'! !

!MaClientServerTestCase methodsFor: 'test UDP object server' stamp: 'cmm 11/10/2008 15:58'!
testUdpObjectLayer
	| sampleObjects1 sampleObjects2 |
	sampleObjects1 := (1 to: 1000) collect: [ : e | MacsTestRequest new ].
	sampleObjects2 := (1 to: 1000) collect: [ : e | MacsTestRequest new ].
	self
		remotePerform: #startUdpObjectServer
			in: 'server' ;
		remotePerform: #sendObjectsToUdpServer:
			with: sampleObjects1
			in: 'client1' ;
		remotePerform: #sendObjectsToUdpServer:
			with: sampleObjects2
			in: 'client2' ;
		remotePerform: #waitUntilDone
			in: 'client1' ;
		remotePerform: #waitUntilDone
			in: 'client2' ;
		remotePerform: #verifyMostObjectsReceived:
			with: sampleObjects1 , sampleObjects2
			in: 'server' ;
		remotePerform: #stopServer
			in: 'server'! !

!MaClientServerTestCase methodsFor: 'test UDP ByteArray server' stamp: 'cmm 11/11/2008 16:52'!
verifyMergedBag
	| shouldBag |
	shouldBag := Bag new.
	{  (500 to: 1000). (750 to: 1250)  } do: 
		[ : eachRange | 
		eachRange first 
			to: eachRange last
			do: 
				[ : n | 
				| words |
				words := n asWords.
				shouldBag add: (words 
						copyFrom: 2
						to: words size - 1) ] ].
	"May we expect at least half of them to get through..?"
	self assert: mergedBag size > (shouldBag size * 0.5) truncated.
	self assert: (shouldBag maIncludesAll: mergedBag)! !

!MaClientServerTestCase methodsFor: 'test UDP object server' stamp: 'cmm 11/10/2008 15:58'!
verifyMostObjectsReceived: aCollection 
	self assert: testObjects size > (aCollection size * 0.8) truncated.
	testObjects do: 
		[ : eachReceivedUdp | 
		self assert: (aCollection anySatisfy: 
				[ : eachTestFixture | 
				eachTestFixture object1 = eachReceivedUdp object1 and: [ eachTestFixture object2 = eachReceivedUdp object2 ] ]) ]! !

!MaClientServerTestCase methodsFor: 'shared' stamp: 'cmm 8/24/2005 20:50'!
waitUntilDone

	[ busy ] whileTrue: [ (Delay forSeconds: 1) wait ].
	^ nil! !

!MaNetworkTestCase class methodsFor: 'initializing' stamp: 'cmm 7/5/2008 14:25'!
addPlayerNamed: aString 
	self 
		addPlayerNamed: aString
		at: (self getHostNameFor: aString)
		port: self getNextPort! !

!MaNetworkTestCase class methodsFor: 'initializing' stamp: 'cmm 2/6/2009 17:26'!
addPlayerNamed: aString at: hostName port: anInteger 
	"The player must first be running and listening on port anInteger."
	self playerLinks 
		at: aString
		put: (MaNetworkTestPlayerFacade 
				name: aString
				host: hostName
				port: anInteger
				testCaseClass: self).
	MaClientServerPreferences debug: true.
	(self playerLinkNamed: aString) timeoutSeconds: 172800	"2 days"! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 2/7/2009 10:15'!
conductorImageFilename
	^ '_' , self suiteName , 'TestConductor'! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 8/24/2005 17:18'!
createAndLaunchImageNamed: aString

	self
		createImageNamed: aString ;
		launchImageNamed: aString! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 2/7/2009 10:09'!
createAndLaunchPlayerImages
	self playersDo: 
		[ : each | 
		self isConductorImage ifTrue: [ self createAndLaunchImageNamed: each imageFilename ] ]! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 8/24/2005 17:18'!
createImageNamed: aString

	| imageName changesName |

	imageName := aString, '.image'.
	MaNotification signal: 'deleting ', imageName.
	FileDirectory default deleteFileNamed: imageName.
	MaNotification signal: 'copying ', SmalltalkImage current imageName, ' to ', imageName.
	FileDirectory default
		copyFileNamed: SmalltalkImage current imageName
		toFileNamed: imageName.

	changesName := aString, '.changes'.
	MaNotification signal: 'deleting ', changesName.
	FileDirectory default deleteFileNamed: changesName.
	MaNotification signal: 'copying ', SmalltalkImage current changesName, ' to ', changesName.
	FileDirectory default
		copyFileNamed: SmalltalkImage current changesName
		toFileNamed: changesName! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 1/13/2010 09:29'!
executableName
	"Florian said SmalltalkImage current getSystemAttribute: 0 does not work for him on his Linux, 
		http://lists.squeakfoundation.org/pipermail/magma/2007-September/000831.html
	so we have this check for Unix."
	^ ((Smalltalk hasClassNamed: #OSProcess) and: [ (Smalltalk classNamed: #OSProcess) isUnix ]) 
		ifTrue: [ (Smalltalk classNamed: #OSProcess) programName ]
		ifFalse: [ SmalltalkImage current getSystemAttribute: 0 ]! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 11/6/2008 17:36'!
getHostNameFor: playerName 
	^ NetNameResolver localHostName

	"Commenting out this old code to allow for one-click run of network test cases."
	"
	^(self players includesKey: playerName)
		ifTrue:
			[ (self playerNamed: playerName) hostname ]
		ifFalse:
			[ NetNameResolver addressFromString: 
				(FillInTheBlank 
					request: 'Enter the hostname of ', playerName
					initialAnswer: 'localhost') ]"! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 5/27/2002 15:16'!
getNextPort

	^NextPort := NextPort + 1
! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 2/6/2009 17:35'!
hasPlayerLinkNamed: aString
	^ self playerLinks includesKey: aString! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 2/6/2009 15:44'!
hasRunningTestPlayer
	^ TestPlayer notNil and: [ TestPlayer isRunning ]! !

!MaNetworkTestCase class methodsFor: 'initializing' stamp: 'cmm 2/11/2009 17:44'!
initialize
	super initialize.
	PlayerLinks := Dictionary new.
	self initializeNextPort! !

!MaNetworkTestCase class methodsFor: 'initializing' stamp: 'cmm 5/27/2002 15:27'!
initializeNextPort

	NextPort := 1313
! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 2/6/2009 13:13'!
isConductorImage
	^ (FileDirectory default localNameFor: SmalltalkImage current imageName) 
		maBeginsWith: self conductorImageFilename
		caseSensitive: true! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 9/12/2007 22:01'!
launchImageNamed: aString
	| imageName |
	imageName := FileDirectory default fullNameFor: aString, '.image'.
	MaNotification signal: 'launching ', imageName.
	(Smalltalk hasClassNamed: #OSProcess)
		ifTrue:
			[ (Smalltalk classNamed: #OSProcess) command: self executableName, ' ', imageName ]
		ifFalse:
			[ Warning signal: 'Cannot launch ', imageName, ' because OSProcess package is not loaded.  Please launch this image yourself before Proceeding.' ]! !

!MaNetworkTestCase class methodsFor: 'private' stamp: 'cmm 12/4/2011 18:08'!
launchPlayerImages
	MaClientServerPreferences debug: true.
	(Smalltalk classNamed: #ScreenController) ifNotNilDo:
		[ : sc | sc new fullScreenOff ].
	"Easier window management in the OS by sizing the windows down.."
	(Smalltalk classNamed: #Display) ifNotNilDo:
		[ : disp | disp class
			depth: Display depth
			width: Display width * 3 // 4
			height: Display height * 3 // 4
			fullscreen: false ].
	(Smalltalk classNamed: #World) ifNotNilDo:
		[ : world | world doOneCycleNow ].
	self
		 saveImageAs: self conductorImageFilename ;
		 createAndLaunchPlayerImages.
	self isConductorImage ifTrue:
		[ self findATranscript.
		self waitForPlayerImages ]! !

!MaNetworkTestCase class methodsFor: 'private' stamp: 'cmm 7/15/2006 10:07'!
maDebugSuite
	"Kickoff each subclass, #maDebug.."
	self withAllSubclassesDo: [ : each | each suite maDebug ]! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 2/6/2009 15:22'!
playerLinkNamed: aString 
	^ self playerLinks at: aString! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 2/6/2009 15:21'!
playerLinks
	^ PlayerLinks! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 2/6/2009 15:22'!
playerLinks: aDictionary 
	"Key is the player name, value is a MaNetworkTestPlayerFacade."
	PlayerLinks := aDictionary! !

!MaNetworkTestCase class methodsFor: 'framework' stamp: 'cmm 2/9/2009 16:36'!
playerPostLaunchInitialization
	"Subclasses override as necessary."! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 2/6/2009 15:21'!
playersDo: aBlock 
	self playerLinks do: aBlock! !

!MaNetworkTestCase class methodsFor: 'initializing' stamp: 'cmm 2/6/2009 15:21'!
removePlayerNamed: aString 
	self playerLinks removeKey: aString! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 8/24/2005 17:18'!
saveImageAs: fileNameString

	SmalltalkImage current 
		closeSourceFiles ;
		saveChangesInFileNamed: (SmalltalkImage current fullNameForChangesNamed: fileNameString) ;
		saveImageInFileNamed: (SmalltalkImage current fullNameForImageNamed: fileNameString)! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 8/26/2005 14:56'!
serverImageFilename
	"All subclasses must override with a unique-name, otherwise the startUp: methods of each will start the test-player server."

	self subclassResponsibility ! !

!MaNetworkTestCase class methodsFor: 'framework' stamp: 'cmm 2/6/2009 12:47'!
setUpPlayers
	self subclassResponsibility ! !

!MaNetworkTestCase class methodsFor: 'framework' stamp: 'cmm 2/6/2009 16:54'!
setUpSuite
	super setUpSuite.
	self
		playerLinks: Dictionary new ;
		initializeNextPort ;
		setUpPlayers ;
"		validate ;
		warn ;"
		launchPlayerImages ;
		playerPostLaunchInitialization.
	^ self isConductorImage! !

!MaNetworkTestCase class methodsFor: 'system startup' stamp: 'cmm 2/6/2009 15:57'!
startPlayer
	self thisPlayer ifNotNilDo: [ : player | self hasRunningTestPlayer ifFalse: [ player startFor: self ] ]! !

!MaNetworkTestCase class methodsFor: 'system startup' stamp: 'cmm 9/2/2010 19:36'!
startUp: resuming 
	resuming ifTrue: [ self startPlayer ]! !

!MaNetworkTestCase class methodsFor: 'framework' stamp: 'cmm 2/6/2009 17:19'!
suiteName
	self subclassResponsibility ! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 2/6/2009 15:48'!
testPlayer: aMaNetworkTestPlayer
	TestPlayer := aMaNetworkTestPlayer! !

!MaNetworkTestCase class methodsFor: 'accessing' stamp: 'cmm 2/7/2009 11:26'!
thisPlayer
	"Answer the player matching my image filename."
	^ self playerLinks 
		detect: 
			[ : each | 
			(each imageFilename , '.image') = (FileDirectory localNameFor: SmalltalkImage current imageName) and: [ self = each testCaseClass or: [ "Magma tests always use a MagmaCollectionTester, needs this" self inheritsFrom: each testCaseClass ] ] ]
		ifNone: [ nil ]! !

!MaNetworkTestCase class methodsFor: 'private' stamp: 'cmm 8/24/2005 17:19'!
validate

	(Smalltalk hasClassNamed: #OSProcess) ifFalse: [ Warning signal: 'OSProcess is recommended to run the test cases with one-click.  Otherwise you will be prompted to launch several images at various times.' ]! !

!MaNetworkTestCase class methodsFor: 'other squeak images' stamp: 'cmm 2/7/2009 11:15'!
waitForPlayerImages
	
	[ (Delay forSeconds: 1) wait.
	self playerLinks allSatisfy: [ : each | each isReady ] ] whileFalse! !

!MaNetworkTestCase class methodsFor: 'private' stamp: 'cmm 2/6/2009 12:35'!
warn
	Warning signal: (String streamContents: 
			[ : stream | 
			stream
				maPrint: 'You are about to kick off network test cases.  To simulate a client-server environment, the suite runs in separate images.  Each of the images is created automatically by saving four copies of this image and its changes file.  One to host the server, one for each client, and one known as the "conductor" image, sends instructions to the others in order to execute the test.

This image is about to be saved under a different name ("' ;
				maPrint: self conductorImageFilename ;
				maPrint: '") and launch a test-suite.  It is therefore recommended to save this image (under its current name) to capture your work before proceeding.		
' ])! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 7/5/2008 14:25'!
getHostNameFor: playerName 
	^ self class getHostNameFor: playerName! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 2/6/2009 15:22'!
playerLinkNamed: aString 
	^ self class playerLinkNamed: aString! !

!MaNetworkTestCase methodsFor: 'accessing' stamp: 'cmm 11/10/2008 14:53'!
playerProtocol
	^ ((MaNetworkTestPlayerRequest allSubclasses
		add: UndefinedObject ;
		yourself) asSortedCollection: [ : a : b | a name < b name ]) asArray! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 5/27/2002 19:33'!
playersDo: aBlock

	self class playersDo: aBlock! !

!MaNetworkTestCase methodsFor: 'printing' stamp: 'cmm 5/3/2002 00:12'!
printOn: aStream

	testSelector
		ifNil: [ aStream nextPutAll: 'a', self class name ]
		ifNotNil: [ super printOn: aStream ]
! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 5/1/2003 17:12'!
pushClass: aClass

	self playersDo: [ :each | each pushClass: aClass ]! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 2/6/2009 15:22'!
remotePerform: selectorSymbol in: playerName 
	^ (self playerLinkNamed: playerName) 
		remotePerform: selectorSymbol
		withArguments: Array new! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 2/6/2009 15:22'!
remotePerform: selectorSymbol with: arg1 in: playerName 
	^ (self playerLinkNamed: playerName) 
		remotePerform: selectorSymbol
		withArguments: (Array with: arg1)! !

!MaNetworkTestCase methodsFor: 'players' stamp: 'cmm 2/6/2009 15:22'!
remotePerform: selectorSymbol withArguments: anArray in: playerName 
	^ (self playerLinkNamed: playerName) 
		remotePerform: selectorSymbol
		withArguments: anArray! !

!MaNetworkTestCase methodsFor: 'private' stamp: 'cmm 2/6/2009 16:49'!
tearDownSuite
	self class isConductorImage ifTrue: [ super tearDownSuite ]! !

!MaTestCase class methodsFor: 'framework' stamp: 'cmm 2/6/2009 16:05'!
setUpSuite
	"Subclasses may implement general set-up code for 
	that should only run once for all tests.  For example,
	network tests maintain a connection to other machines
	that participate in the test.  The facades to these other
	machines needs to be maintained."
	^ true! !

!MaTestCase class methodsFor: 'testing' stamp: 'cmm 4/18/2002 00:11'!
shouldInheritSelectors

	^false! !

!MaTestCase class methodsFor: 'framework' stamp: 'cmm 4/8/2005 13:05'!
tearDownSuite
	"Counterpart to setUpSuite.  See that method for
	more info."

	Transcript cr; show: 'Tests complete.  Tearing down suite..'! !

!MaTestCase methodsFor: 'accessing' stamp: 'cmm 5/13/2010 18:53'!
defaultTimeout 
	^ 500 days asSeconds! !

!MaTestCase methodsFor: 'file management' stamp: 'cmm 12/5/2002 23:11'!
deleteFilesNamed: aCollection

	aCollection do:
		[ :each |
		StandardFileStream allInstances do:
			[ :eachStream |
			eachStream fullName = each ifTrue:
				[ self notify: 'closing a rogue FileStream for ', eachStream fullName.
				eachStream close ] ].
		self notify: 'deleting file ', each.
		(FileDirectory forFileName: each) deleteFileNamed: each ]! !

!MaTestCase methodsFor: 'accessing' stamp: 'cmm 9/11/2008 19:44'!
environmentDirectory

	^ FileDirectory default! !

!MaTestCase methodsFor: 'accessing' stamp: 'cmm 10/7/2004 16:59'!
environmentPath

	^ self environmentDirectory pathName! !

!MaTestCase methodsFor: 'running' stamp: 'cmm 4/18/2002 00:54'!
maDebug: aTestResult

	aTestResult maDebugCase: self
! !

!MaTestCase methodsFor: 'running' stamp: 'cmm 4/18/2002 00:56'!
maDebugAsFailure: aTestResult

	aTestResult maDebugCaseAsFailure: self
! !

!MaTestCase methodsFor: 'private' stamp: 'cmm 8/19/2007 22:10'!
notify: aString 
	MaArmoredCodeNotification signal: DateAndTime now printString , ':  ' , aString! !

!MaTestCase methodsFor: 'private' stamp: 'cmm 8/19/2007 22:08'!
performTest
	[ self
		notify: 'running ' , self selector , '.. ' ;
		notify: 'time to run ' , self selector , ':  ' , [ super performTest ] durationToRun printString ] 
		on: MaNotification
		do: 
			[ : n | 
			Transcript
				cr ;
				show: n messageText.
			n resume ]! !

!MaTestCase methodsFor: 'accessing' stamp: 'cmm 10/7/2004 17:13'!
projectDirectory
	"The projectDirectory typically resides in the projectsDirectory, with my subclasses typically appending their project name to the projectsDirectory."

	self subclassResponsibility! !

!MaTestCase methodsFor: 'accessing' stamp: 'cmm 10/7/2004 17:27'!
projectsDirectory

	^ (FileDirectory default directoryExists: 'My projects')
		ifTrue:
			[ FileDirectory on: (FileDirectory default fullNameFor: 'My projects') ]
		ifFalse:
			[ FileDirectory default ]! !

!MaTestCase methodsFor: 'accessing' stamp: 'cmm 10/7/2004 17:26'!
resourcesDirectory

	^ (self projectDirectory directoryExists: 'Resources')
		ifTrue:
			[ FileDirectory on: (self projectDirectory fullNameFor: 'Resources') ]
		ifFalse: [ self projectDirectory ]! !

!MaTestCase methodsFor: 'private' stamp: 'cmm 2/6/2009 16:47'!
setUpSuite
	"Subclasses may implement general set-up code for that should only run once for all tests.  For example,network tests maintain a connection to other machinesthat participate in the test.  The facades to these other machines needs to be maintained."
	^ self class setUpSuite! !

!MaTestCase methodsFor: 'private' stamp: 'cmm 12/28/2008 23:09'!
tearDownSuite
	self class tearDownSuite! !

PackageInfo subclass: #MaArmoredCodePackageInfo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Info'!

!MaArmoredCodePackageInfo class methodsFor: 'as yet unclassified' stamp: 'cmm 4/21/2011 16:42'!
initialize
	^ self new register! !

MaNotification subclass: #MaArmoredCodeNotification
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Exceptions'!

MaSingleton subclass: #MaArmoredCodeOptions
	instanceVariableNames: 'testEnvironmentDirectory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-API'!

!MaArmoredCodeOptions class methodsFor: 'initializing' stamp: 'cmm 10/6/2004 16:40'!
initialize

	self reset! !

!TestSuite methodsFor: '*ma-client-server-tester' stamp: 'cmm 2/8/2009 22:48'!
addTestsFirst: aCollection 
	"Add aCollection of tests to the beginning of my tests collection."
	self tests addAllFirst: aCollection! !

!TestSuite methodsFor: '*ma-client-server-tester' stamp: 'cmm 4/18/2002 00:39'!
maDebug

	self maRunUsing: #maDebug:
! !

!TestSuite methodsFor: '*ma-client-server-tester' stamp: 'cmm 4/18/2002 00:41'!
maDebug: aResult 

	self
		maExecuteTestsUsing: #maDebug:
		result: aResult
! !

!TestSuite methodsFor: '*ma-client-server-tester' stamp: 'cmm 4/18/2002 00:43'!
maDebugAsFailure

	self maRunUsing: #maDebugAsFailure:
! !

!TestSuite methodsFor: '*ma-client-server-tester' stamp: 'cmm 4/18/2002 00:42'!
maDebugAsFailure: aResult 

	self
		maExecuteTestsUsing: #maDebugAsFailure:
		result: aResult
! !

!TestSuite methodsFor: '*ma-client-server-tester' stamp: 'cmm 2/16/2009 15:23'!
maExecuteTestsUsing: aSymbol result: aResult 
	self tests isEmpty ifTrue: [ ^ self ].
	
	[ | shouldRunTests |
	shouldRunTests := [ self tests anyOne setUpSuite ] 
		on: MaNotification
		do: 
			[ : noti | 
			Transcript
				cr ;
				show: noti messageText.
			noti resume ].
	shouldRunTests ifTrue: 
		[ | durationToRun |
		durationToRun := 
		[ self tests do: 
			[ : each | 
			self changed: each.
			each 
				perform: aSymbol
				with: aResult ] ] durationToRun.
		Transcript
			cr ;
			show: 'Time to run all tests:' , durationToRun asString ] ] ensure: 
		[ [ self tests anyOne tearDownSuite ] 
			on: MaNotification
			do: 
				[ : noti | 
				Transcript
					cr ;
					show: noti messageText.
				noti resume ].
		Transcript
			cr ;
			show: 'Done.' ]! !

!TestSuite methodsFor: '*ma-client-server-tester' stamp: 'cmm 9/16/2006 18:57'!
maRunUsing: aSymbol
	| result |
	result := TestResult new.
	[ self 
		perform: aSymbol 
		with: result ] 
			ensure: [self resources do: [:each | each reset ] ].
	^ result! !

!TestResult methodsFor: '*ma-client-server-tester' stamp: 'cmm 4/18/2002 00:54'!
maDebugCase: aTestCase

	aTestCase runCase.
	self passed add: aTestCase
! !

!TestResult methodsFor: '*ma-client-server-tester' stamp: 'cmm 4/18/2002 00:56'!
maDebugCaseAsFailure: aTestCase

	aTestCase halt runCase.
	self passed add: aTestCase
! !

MaWarning subclass: #MaArmoredCodeWarning
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Exceptions'!

MaClientServerRequest subclass: #MaNetworkTestPlayerRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-private'!

!MaNetworkTestPlayerRequest commentStamp: 'cmm 8/18/2009 21:40' prior: 0!
An abstract class for sending requests from the test conductor to the test 'players'.!

!MaNetworkTestPlayerRequest methodsFor: 'as yet unclassified' stamp: 'cmm 12/23/2002 17:42'!
processFor: aMaNetworkTestPlayer

	self subclassResponsibility! !

MaNetworkTestPlayerRequest subclass: #MaPushCodeRequest
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-private'!

!MaPushCodeRequest methodsFor: 'private' stamp: 'cmm 6/7/2004 23:53'!
processFor: aMaNetworkTestPlayer

	aMaNetworkTestPlayer loadCodeFrom: self stream.
	^ nil! !

!MaPushCodeRequest methodsFor: 'accessing' stamp: 'cmm 4/27/2002 15:16'!
stream

	^stream
! !

!MaPushCodeRequest methodsFor: 'accessing' stamp: 'cmm 4/27/2002 15:16'!
stream: aPositionableStream

	stream := aPositionableStream! !

MaNetworkTestPlayerRequest subclass: #MaRemotePerformRequest
	instanceVariableNames: 'selector arguments'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-private'!

!MaRemotePerformRequest commentStamp: 'cmm 8/18/2009 21:41' prior: 0!
Perform #selector in the TestCase subclass instance of a remote test-player image.!

!MaRemotePerformRequest methodsFor: 'accessing' stamp: 'cmm 5/6/2002 23:28'!
arguments

	^arguments
! !

!MaRemotePerformRequest methodsFor: 'accessing' stamp: 'cmm 5/6/2002 23:28'!
arguments: aSymbol

	arguments := aSymbol
! !

!MaRemotePerformRequest methodsFor: 'private' stamp: 'cmm 4/27/2002 16:33'!
processFor: aMaNetworkTestPlayer

	^aMaNetworkTestPlayer
		remotePerform: self selector
		withArguments: self arguments
! !

!MaRemotePerformRequest methodsFor: 'accessing' stamp: 'cmm 4/27/2002 15:07'!
selector

	^selector
! !

!MaRemotePerformRequest methodsFor: 'accessing' stamp: 'cmm 4/27/2002 15:07'!
selector: aSymbol

	selector := aSymbol
! !

MaNetworkTestPlayerRequest subclass: #MaSetTestCaseClassRequest
	instanceVariableNames: 'testCaseClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-private'!

!MaSetTestCaseClassRequest methodsFor: 'private' stamp: 'cmm 6/7/2004 23:59'!
processFor: aMaNetworkTestPlayer

	aMaNetworkTestPlayer setTestCaseClass: self testCaseClass.
	^ nil! !

!MaSetTestCaseClassRequest methodsFor: 'accessing' stamp: 'cmm 5/27/2002 20:15'!
testCaseClass

	^testCaseClass! !

!MaSetTestCaseClassRequest methodsFor: 'accessing' stamp: 'cmm 5/27/2002 20:15'!
testCaseClass: aClass

	testCaseClass := aClass! !

MaClientServerRequest subclass: #MacsTestRequest
	instanceVariableNames: 'object1 object2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Object Layer Tests'!

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 11:29'!
initialize
	super initialize.
	object1 := DateAndTime 
		date: 
			(Date 
				newDay: (1 to: 365) atRandom
				year: (1900 to: 2100) atRandom)
		time: Time now.
	object2 := UUID new -> (1 to: 100000) atRandom asWords! !

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 10:57'!
object1
	^object1! !

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 10:57'!
object1: aDateAndTime
	object1 := aDateAndTime! !

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 10:57'!
object2
	^object2! !

!MacsTestRequest methodsFor: 'accessing' stamp: 'cmm 11/9/2008 10:58'!
object2: anAssociation
	object2 := anAssociation! !

MaObject subclass: #MaNetworkTestPlayer
	instanceVariableNames: 'testCase server'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Network testing'!

!MaNetworkTestPlayer class methodsFor: 'as yet unclassified' stamp: 'cmm 7/2/2002 00:13'!
testCaseClass: aNetworkTestCaseClass

	^self new 
		setTestCaseClass: aNetworkTestCaseClass;
		yourself! !

!MaNetworkTestPlayer methodsFor: 'testing' stamp: 'cmm 2/6/2009 14:02'!
isRunning
	^ server notNil and: [ server isRunning ]! !

!MaNetworkTestPlayer methodsFor: 'access' stamp: 'cmm 6/7/2004 23:53'!
loadCodeFrom: aPositionableStream

	aPositionableStream position: 0.
	aPositionableStream 
		maFileInAnnouncing: 'loading code from conductor..' 
		logged: false! !

!MaNetworkTestPlayer methodsFor: 'request framework requirements' stamp: 'cmm 6/22/2004 10:37'!
processRequest: aMaNetworkTestPlayerRequest

	| result |
	self maMarked: 'magic'.  "This method called as part of test-script user-setup.  See MagmaTesterPackageInfo>>#readmeText."
	MaArmoredCodeNotification signal: 'Processing ', aMaNetworkTestPlayerRequest printString.
	result := aMaNetworkTestPlayerRequest processFor: self.
	^ result == testCase ifFalse: [ result ]! !

!MaNetworkTestPlayer methodsFor: 'access' stamp: 'cmm 10/30/2004 17:27'!
protocol

	^ testCase playerProtocol! !

!MaNetworkTestPlayer methodsFor: 'access' stamp: 'cmm 6/7/2004 21:55'!
remotePerform: selector withArguments: anArray

	| result |
	result := testCase
		perform: selector
		withArguments: anArray.
	^ (result == testCase or: [ result == self ])
		ifTrue: [ nil ]
		ifFalse: [ result ]! !

!MaNetworkTestPlayer methodsFor: 'access' stamp: 'cmm 2/6/2009 15:50'!
server: aMaNetworkRequestServer 
	server := aMaNetworkRequestServer! !

!MaNetworkTestPlayer methodsFor: 'access' stamp: 'cmm 6/6/2004 13:24'!
setTestCaseClass: aClass

	testCase := aClass new! !

MaObject subclass: #MaNetworkTestPlayerFacade
	instanceVariableNames: 'link name testCaseClass'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-private'!

!MaNetworkTestPlayerFacade class methodsFor: 'creation' stamp: 'cmm 2/6/2009 17:26'!
name: aString host: hostName port: anInteger testCaseClass: aClass 
	^ self new
		name: aString ;
		testCaseClass: aClass ;
		link: (MaNetworkServerLink location: (MaServerLocation 
					host: hostName
					port: anInteger)) ;
		yourself! !

!MaNetworkTestPlayerFacade methodsFor: 'initializing' stamp: 'cmm 5/2/2002 23:58'!
connect

	link connect
! !

!MaNetworkTestPlayerFacade methodsFor: 'initializing' stamp: 'cmm 1/17/2009 16:08'!
disconnect
	link disconnect! !

!MaNetworkTestPlayerFacade methodsFor: 'access' stamp: 'cmm 2/7/2009 10:16'!
imageFilename
	^ String streamContents: 
		[ : stream | 
		stream
			maPrint: $_ ;
			maPrint: testCaseClass suiteName ;
			maPrint: 'Test' ;
			maPrint: self name ]! !

!MaNetworkTestPlayerFacade methodsFor: 'testing' stamp: 'cmm 6/16/2012 12:14'!
isReady
	| socket |
	^ [ socket := Socket newTCP
		connectTo: NetNameResolver localHostAddress
		port: link port
		waitForConnectionFor: 5.
	socket closeAndDestroy.
	true ]
		on: ConnectionTimedOut, ConnectionRefused
		do:
			[ : err | MaArmoredCodeNotification signal: 'Could not connect to port ' , link port asString , ', waiting...'.
			false ]! !

!MaNetworkTestPlayerFacade methodsFor: 'access' stamp: 'cmm 3/5/2009 22:39'!
link
	^ link! !

!MaNetworkTestPlayerFacade methodsFor: 'initializing' stamp: 'cmm 4/27/2002 14:50'!
link: aMaServerLink

	link := aMaServerLink! !

!MaNetworkTestPlayerFacade methodsFor: 'access' stamp: 'cmm 5/22/2002 10:21'!
name

	^name
! !

!MaNetworkTestPlayerFacade methodsFor: 'initializing' stamp: 'cmm 5/8/2002 22:00'!
name: aString

	name := aString! !

!MaNetworkTestPlayerFacade methodsFor: 'private' stamp: 'cmm 5/22/2002 17:32'!
pushClass: aClass

	| stream |
	stream := ReadWriteStream on: String new.
	aClass
		fileOutOn: stream
		moveSource: false
		toFile: 0.
	^self pushCode: stream
! !

!MaNetworkTestPlayerFacade methodsFor: 'private' stamp: 'cmm 3/21/2003 00:55'!
pushCode: aPositionableStream

	MaArmoredCodeNotification signal: 'pushing code to ', name.
	^self submit:
		(MaPushCodeRequest new
			stream: aPositionableStream;
			yourself)! !

!MaNetworkTestPlayerFacade methodsFor: 'access' stamp: 'cmm 3/21/2003 00:55'!
remotePerform: selectorSymbol withArguments: anArray

	MaArmoredCodeNotification signal: 'remote performing ', selectorSymbol, ' with arguments ', anArray printString, ' in ', name.
	^self submit:
		(MaRemotePerformRequest new
			selector: selectorSymbol;
			arguments: anArray)! !

!MaNetworkTestPlayerFacade methodsFor: 'player access' stamp: 'cmm 3/21/2003 00:55'!
setTestCaseClass: aClass
	"Make sure you have first updated the ChangeSet that contains your test case class, or the player may not be able to instantiate it."

	MaArmoredCodeNotification signal: 'setting test case class to ', aClass name, ' in ', name.
	^self submit:
		(MaSetTestCaseClassRequest new
			testCaseClass: aClass;
			yourself)
! !

!MaNetworkTestPlayerFacade methodsFor: 'private' stamp: 'cmm 5/10/2011 17:54'!
startFor: testCaseClass 
	"Start the server for this player."
	| server testPlayer |
	testPlayer := MaNetworkTestPlayer testCaseClass: testCaseClass.
	server := MaNetworkRequestServer protocol: testPlayer protocol.
	server console: MaTranscriptConsole new.
	server 
		processOn: link port
		using: [ : req | testPlayer processRequest: req ].
	testCaseClass testPlayer: 
		(testPlayer
			server: server ;
			yourself)! !

!MaNetworkTestPlayerFacade methodsFor: 'private' stamp: 'cmm 1/2/2003 14:46'!
submit: aMaNetworkTestPlayerRequest

	| result |
	result := link submit: aMaNetworkTestPlayerRequest.
	result maIsException 
		ifTrue: [ result class signal: result messageText ]
		ifFalse: [ ^result ]! !

!MaNetworkTestPlayerFacade methodsFor: 'access' stamp: 'cmm 2/7/2009 10:52'!
testCaseClass
	^ testCaseClass! !

!MaNetworkTestPlayerFacade methodsFor: 'initializing' stamp: 'cmm 2/6/2009 17:27'!
testCaseClass: aClass 
	testCaseClass := aClass! !

!MaNetworkTestPlayerFacade methodsFor: 'initializing' stamp: 'cmm 10/16/2002 17:04'!
timeoutSeconds: anInteger

	link timeoutSeconds: anInteger! !

MaObject subclass: #MacsTestResponse
	instanceVariableNames: 'response1 response2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Ma-Client-Server-Tester-Object Layer Tests'!

!MacsTestResponse class methodsFor: 'as yet unclassified' stamp: 'cmm 11/9/2008 11:10'!
fromRequest: aMacsTestRequest 
	^ self new
		initializeFrom: aMacsTestRequest ;
		yourself! !

!MacsTestResponse methodsFor: 'as yet unclassified' stamp: 'cmm 11/9/2008 17:35'!
initializeFrom: aMacsTestRequest 
	response1 := DateAndTime 
		date: (aMacsTestRequest object1 asDate addDays: 1)
		time: aMacsTestRequest object1 asTime.
	response2 := aMacsTestRequest object2 value -> aMacsTestRequest object2 key! !

!MacsTestResponse methodsFor: 'accessing' stamp: 'cmm 11/9/2008 11:14'!
response1
	^response1! !

!MacsTestResponse methodsFor: 'accessing' stamp: 'cmm 11/9/2008 11:14'!
response2
	^response2! !
MaNetworkTestCase initialize!
MaArmoredCodePackageInfo initialize!
MaArmoredCodeOptions initialize!
MaClientServerTestCase initialize!
